/**
 *
 * @param repository
 * @param options
 * @returns
 */
export function handleSession(repository, options = {}) {
    return async function ({ event, resolve }) {
        event.locals.session = new SveltekitSession(event, repository, options);
        return resolve(event);
    };
}
function defaultSessionKey() {
    return crypto.randomUUID();
}
export class SveltekitSession {
    event;
    repository;
    options;
    sessionKey;
    constructor(event, repository, options = {}) {
        this.event = event;
        this.repository = repository;
        this.options = {
            keyGenerator: options.keyGenerator ?? defaultSessionKey,
            sessionCookieName: options.sessionCookieName ?? 'svelte-session-key',
            debug: options.debug ?? false,
            secure: options.secure ?? true,
        };
        this.repository.purgeExpired();
        this.sessionKey = event.cookies.get(this.options.sessionCookieName) ?? null;
        if (this.sessionKey && !this.repository.exists(this.sessionKey)) {
            this.event.cookies.delete(this.options.sessionCookieName, {
                path: '/',
                httpOnly: true,
                maxAge: this.repository.ttl().toSecond(),
                secure: this.options.secure,
            });
            this.sessionKey = null;
        }
    }
    log(message, ...params) {
        if (this.options.debug) {
            console.log(message, ...params);
        }
    }
    async create() {
        if (this.sessionKey) {
            return false;
        }
        this.sessionKey = this.options.keyGenerator();
        this.log(`Creating new ${this.repository.name} session ${this.sessionKey}`);
        this.event.cookies.set(this.options.sessionCookieName, this.sessionKey, {
            path: '/',
            httpOnly: true,
            maxAge: this.repository.ttl().toSecond(),
            secure: this.options.secure,
        });
        await this.repository.initializeSession(this.sessionKey);
        return true;
    }
    async exists() {
        if (!this.sessionKey) {
            return false;
        }
        return this.repository.exists(this.sessionKey);
    }
    async get(initializeIfNull = false) {
        if (!this.sessionKey) {
            if (initializeIfNull) {
                this.create();
            }
            return null;
        }
        return this.repository.loadValue(this.sessionKey);
    }
    async update(value) {
        if (!this.sessionKey) {
            return false;
        }
        this.event.cookies.set(this.options.sessionCookieName, this.sessionKey, {
            path: '/',
            httpOnly: true,
            maxAge: this.repository.ttl().toSecond(),
            secure: this.options.secure,
        });
        await this.repository.storeValue(this.sessionKey, value);
        this.log(`Updating ${this.repository.name} session ${this.sessionKey}`, value);
        return true;
    }
    async destroy() {
        if (!this.sessionKey) {
            return false;
        }
        this.event.cookies.delete(this.options.sessionCookieName, {
            path: '/',
            httpOnly: true,
            maxAge: this.repository.ttl().toSecond(),
            secure: this.options.secure,
        });
        await this.repository.destroySession(this.sessionKey);
        this.log(`Destroying ${this.repository.name} session ${this.sessionKey}`);
        return true;
    }
}
